package data

/**
 * A Lisply-named representation of a singly-linked list and an analogue to
 * Scala's immutable List. This is a special case of any type A where there
 * exists a nondeterministic selection of its values that must be specially
 * handled such that the number of values is inconsequential.
 */
sealed trait LispList[+A] {

  def car: A

  def cdr: LispList[A]
}

case class LispCons[+A](car: A, cdr: LispList[A]) extends LispList[A]

case object LispNil extends LispList[Nothing] {

  def car: Nothing = throw new Exception("LispList with no car!")

  def cdr: LispList[Nothing] = throw new Exception("LispList with no cdr!")
}

object LispList {

  def apply[A](): LispList[A] = LispNil

  def apply[A](items: A*): LispList[A] = items.foldRight(LispList[A]())(LispCons[A])

  // TODO how to make a Seq
  // def unapplySeq[A](list: LispList[A]): Option[Seq[A]] = Some(list.toSeq)
}
