package data

/**
 * Superposition is an analogue to Scala's Either and \a special case of a
 * product (any type of A × B) where only A or B exists, but must be treated
 * specially as though it could be either one.
 */
sealed trait Superposition[+A, +B]

/**
 * Implied negative case of the Superposition. This is probably not the one you
 * want.
 *
 * @param value The thing you probably don't want.
 * @tparam A Downside type
 * @tparam B Upside type
 */
case class Downside[+A, +B](value: A) extends Superposition[A, B]

/**
 * Implied positive case of the Superposition. This is usually the one you want.
 *
 * @param value The thing you usually want.
 * @tparam A Downside type
 * @tparam B Upside type
 */
case class Upside[+A, +B](value: B) extends Superposition[A, B]
