package object data {

  /**
   * A modeled stateful computation that for any state of type S there exists
   * a computation that derives from S a product of type A and a successive
   * instance of S representing the state of the next computation.
   *
   * @tparam S The state type
   * @tparam A The value returned by the operation
   */
  type State[S, +A] = S => (S, A)

  /**
   * A special case of any type A that is just itself.
   */
  type Identity[+A] = A
}
