package typeclasses

/**
 * Defines the ApplicativeError operations for any type of kind F[_] for error
 * type E. ApplicativeError is a special case of Applicative where the type
 * kind of F[_] may represent a superposition of "success" or "error" cases
 * and affords operations to create and recover these error cases.
 */
trait ApplicativeError[F[_], E] extends Applicative[F] {

  /**
   * Lift an error handler into F[_] to apply to E.
   */
  def handleError[A](fa: F[A])(f: E => A): F[A]

  /**
   * Lift an error handler into F[_] to apply to E, returning a lifted result.
   */
  def handleErrorWith[A](fa: F[A])(f: E => F[A]): F[A]

  /**
   * A special case of pure() which lifts an error E into F[_].
   */
  def raiseError[A](e: E): F[A] = ???
}
