package typeclasses

/**
 * Defines the Monad operations for any type of kind F[_]. A Monad is a special
 * case of Applicative where the application of a function to a lifted value
 * in F[_] may itself return a lifted value. This is especially powerful because
 * the lifted result may represent a case against which no further operations
 * may be performed, which can model short-circuiting on errors!
 */
trait Monad[F[_]] extends Applicative[F] {

  /**
   * Lift an apply a function into F[_] and apply it to the value. The function
   * returns its own lifted result.
   */
  def flatMap[A, B](fa: F[A])(f: A => F[B]): F[B]
}
